
//*****************************************************************************
//
// File Name	: 'SED1530.c'
// Title		: SPI interface driver for the E0855-2 SED1530 graphic LCD
// Author		: Marc Meise
// Created		: Oktober 2007
// Version		: Beta
// Target MCU	: Atmel AVR series
//
//
// This code is distributed under the GNU Public License
// which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

/*  
	Disclaimer:
	This code is highly beta and might lack some funcionality and/or documentation.
	Furthermore the code has not been properly tested.
	Nevertheless, please expect most functions to work.

	I will not take any responsibility or what so ever for the functionality of the E0855-2 SED1530 driver.
	If you wreck anything - it's your own business...
	Modify this code and contribute freely.
*/

#include <avr/io.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include <font.h>
#include <string.h>
#include <PICS.h>

// Port-definitions
// Alter the Port-definitions to your convenience.

#define LCDPORT		PORTB				//PORT where the GLCD ist connected
#define A0			PB0					//A0
#define RES 		PB3					//Reset
#define CS1 		PB1					//CS1 is Low-active
#define CS2 		PB2					//CS2 is high-active
#define SS  		PB4					//Pinnumber of /SS, will be GPIO (turned to output)
#define MOSI		PB6					//MOSI
#define SCK			PB7					//SCK


// Only alter the following code, if you know what you're doing...

#define COL_Hi		0b00010001			//Column 16 HI is left for E0855-2			
#define COL_Lo		0b00000000			//Column 00 Lo is left for E0855-2
#define SC_width	96					//Screenwidth (96 for the E0855-2)

#define A0_Hi		PORTB|=(1<<A0)		//Change A0
#define A0_Lo	  	PORTB&=~(1<<A0)
#define RES_On  	PORTB&=~(1<<RES)	//Reset
#define RES_Off  	PORTB|=(1<<RES)


// Functions
void Init_SPI (void);
void SPI_out (uint8_t Data);
void Init_SED1530 (void);
void SED_clear (void);
void SED_clear_page (uint8_t Page);
void SED_Chg_page (uint8_t Page);
void SED_Home_Col (void);
void Goto_XY (uint8_t X, uint8_t Y);
void SED_putc (char character);
void SED_puts (char *string);
void SED_symbol(uint8_t symbol, uint8_t OnOff);
void SED_newline(void);
void SED_showPIC (uint8_t picture);
void SED_puts_num (uint16_t numeric);
uint8_t ReverseBitorder (uint8_t Bits);
void SED_set_XY(uint8_t Page, uint8_t Column);

//some Variables
volatile uint8_t data;							
volatile int pagecounter=0, columncounter=0;
volatile uint8_t font[127][6];

//a little delay
void delay (void)
{
 volatile unsigned char a,b;
 for (a=0;a<=254;a++)
 	{
	 for (b=0;b<=254;b++)
	 	{
		 asm volatile("NOP");
		 asm volatile("NOP");
		 asm volatile("NOP");
		 asm volatile("NOP");
		 asm volatile("NOP");
	 	}
	 }
}


//Initialize SPI
void Init_SPI (void)				
{
 delay();									// wait a little
 LCDPORT|=(1<<MOSI)|(1<<SCK)|(1<<SS);
 SPCR|= (1<<MSTR);						//Set SPI-Master mode
 SPCR|= (1<<SPR0);						//Set Clock/16
 SPCR&=~(1<<SPR1);				
 SPCR&=~(1<<CPOL);						//Phase positiv
 SPCR&=~(1<<DORD);						//Data-Order MSB
 SPCR&=~(1<<SPIE);						//Interrupt not used
 SPCR|= (1<<CPHA);						//Clock-Phase High 
 SPCR|= (1<<SPE);						//Enable SPI
}

//send one byte over SPI
void SPI_out (uint8_t Data)
{
 SPDR=Data;
 while(!(SPSR&(1<<SPIF)))
 ;
}

//Initialize the SED1530 (E0855-2)
void Init_SED1530 (void)
{
 Init_SPI();								//Initialise the SPI
 RES_On;									// RESET the SED
 LCDPORT&=~(1<<CS1);						// /CS1 goes LOW
 LCDPORT|= (1<<CS2);						// CS2 goes HI
 delay();
 delay();
 delay();
 delay();
 RES_Off;									// SED is resetted
 A0_Lo;										// Enter SED1530 command-mode
 delay();
 SPI_out(0b01000000);						// Start display Line 0
 SPI_out(0b10100001);						// ADC inverse direction
 SPI_out(0b10100010);						// LCD-Bias= 1/5
 SPI_out(0b11000000);						// Output Status 0
 SPI_out(0b00101111);						// GLCD Voltage-Regulators ON
 SPI_out(0b10001111);						// Contrast=Medium
 SPI_out(0b10100100);						// Display-Test OFF
 SPI_out(0b10101111);						// Display ON
 SPI_out(0b10100110);						// Display NOT inverted
 SPI_out(0b10110011);						// Set Page 3						
 SPI_out(COL_Hi);							// Set Column by Hi 4-Bits															
 SPI_out(COL_Lo);							// Set Column by Low 4-Bits
 A0_Hi;										// Enter SED1530 data-mode										 
}

// Clear Screen
void SED_clear (void)
{
 uint8_t pagecount, columncounter;
 for (pagecount=0; pagecount<=8; pagecount++)		//Clear Display by erasing all Pixels
 	{
	 SED_Chg_page(pagecount);						//Set appropriate Page
	 SED_Home_Col();								//Set left column
	 for(columncounter=0;columncounter<=96;columncounter++)
	 	{
		 SPI_out(0x00);
		}
	}
 SED_Chg_page(0b00000011);								//Change to Page 3
 SED_Home_Col();										//Set left column
}

// Clear Page
void SED_clear_page (uint8_t Page)
{
 uint8_t columncounter;
 SED_Chg_page(Page);									//Set appropriate Page
 SED_Home_Col();										//Set left column
 for (columncounter=0;columncounter<=96;columncounter++)
 	{
	 SPI_out(0x00);
	}
 SED_Home_Col();										//Set left column
}
//Change Page
void SED_Chg_page (uint8_t Page)
{
 A0_Lo;
 switch (Page)
	{
	 case (0) : SPI_out(0b10110000); break;		//Set Page 0
	 case (1) : SPI_out(0b10110001); break;		//Set Page 1
	 case (2) : SPI_out(0b10110010); break;		//Set Page 2
	 case (3) : SPI_out(0b10110011); break;		//Set Page 3
	 case (8) : SPI_out(0b10111000); break;		//Set Page 8 (Special characters for The E0855-2)
	}
 pagecounter=Page;
 A0_Hi;
}


//Set the Left visible Corner of the Display
void SED_Home_Col (void)
{
 A0_Lo;											// Enter SED1530 command-mode
 SPI_out(COL_Hi);								// Set Column by Hi and Low 4-Bits
 SPI_out(COL_Lo);
 columncounter=0;
 A0_Hi;											// Enter SED1530 data-mode
}

//Display one Character
void SED_putc (char character)
{
 uint8_t column;
 if (character>31)
 	{
 	 for(column=0;column<=5;column++)
 		{
		 if (columncounter>=96)
			{
			 SED_newline();	
			}
		 if (pagecounter<0)
			{
			 pagecounter=3;
			 SED_Chg_page(pagecounter);
			 SED_Home_Col ();	
			}	 	
 	 	 data=pgm_read_byte(&font[character-32][column]);
		 data=ReverseBitorder(data);
 	 	 SPI_out(data);
		 columncounter++;
		}	
	}
}

//reverse the Bit-order for inverse Display of the E0855-2
//This is the only function, which is not my coding.
//Respect and honor to Mr. Huslik for this great function. 
uint8_t ReverseBitorder (uint8_t Bits)
{
  uint8_t ReversedBit;
  ReversedBit = (Bits >> 4) | (Bits << 4);
  ReversedBit = ((ReversedBit & 0xcc) >> 2 ) | ((ReversedBit & 0x33) << 2);
  ReversedBit = ((ReversedBit & 0xaa) >> 1 ) | ((ReversedBit & 0x55) << 1);
  return ReversedBit;
}

//display character-String
void SED_puts (char *string)
{
 while (*string)
 {
  SED_putc(*string);
  string++;
 }
}

//display numeric-Values
void SED_puts_num (uint16_t numeric)
{
 int buffer[20];
 itoa(numeric,buffer,10);
 SED_puts(buffer);
}

//Show a picture out of the picture-file
void SED_showPIC (uint8_t picture)
{ 
 uint16_t column;
 SED_Chg_page(3);
 SED_Home_Col();
  	 for(column=0;column<=383;column++)
 		{
		 switch (picture)
 			{
	 		case (0): data=pgm_read_byte(&picture0[column]);; break;	
	 		case (1): data=pgm_read_byte(&picture1[column]);; break;
	 		case (2): data=pgm_read_byte(&picture2[column]);; break;
	 		case (3): data=pgm_read_byte(&picture3[column]);; break;
	 		case (4): data=pgm_read_byte(&picture4[column]);; break;
	 		case (5): data=pgm_read_byte(&picture5[column]);; break;
	 		case (6): data=pgm_read_byte(&picture6[column]);; break;
	 		case (7): data=pgm_read_byte(&picture7[column]);; break;
	 		case (8): data=pgm_read_byte(&picture8[column]);; break;
	 		case (9): data=pgm_read_byte(&picture9[column]);; break;
			}		 
		 data= ReverseBitorder(data);
 	 	 SPI_out(data);
		 switch (column)
		 	{
			 case (SC_width-1) : SED_Chg_page(2); SED_Home_Col(); break;
			 case ((SC_width-1)+SC_width): SED_Chg_page(1); SED_Home_Col(); break;
			 case ((SC_width-1)+(SC_width*2)): SED_Chg_page(0); SED_Home_Col(); break;
			}
		}
 SED_Chg_page(3);
 SED_Home_Col();	  
}

//Access to the Symboles of the E2855-2
void SED_symbol(uint8_t symbol, uint8_t OnOff)
{
 unsigned char temp;
 temp=pagecounter;
 SED_set_XY(8,symbol-16);
 if (OnOff>=1)
 	{
	 SPI_out(0x01);
    }
	else
		{
		 SPI_out(0x00);
		}
 SED_Chg_page(temp);
 SED_Home_Col();
 }

//Newline and carriage return
void SED_newline()
{
 pagecounter=pagecounter-1;
 	if (pagecounter<0)
	{
	 pagecounter=3;
	 SED_Chg_page(pagecounter);
	 SED_Home_Col ();	
	}
	else
	{
 	 SED_Chg_page(pagecounter);
 	 SED_Home_Col ();
	}
}

//Select Page and Column
void SED_set_XY(uint8_t Page, uint8_t Column)
{
 SED_Home_Col ();
 SED_Chg_page (Page);
 unsigned char x;
 Column=Column+16;
 volatile unsigned char col_HI=Column,col_LO=Column;
 columncounter=Column-16;
 A0_Lo;										
 for (x=0;x<=3;x++)
 	{
	 col_HI=col_HI>>1;
	 col_LO=col_LO<<1;
	}
 for (x=0;x<=3;x++)
 	{
	 col_LO=col_LO>>1;
	}
 col_HI=col_HI+16;
 SPI_out(col_HI);
 SPI_out(col_LO);
 A0_Hi;
}




