
//*****************************************************************************
//
// File Name	: 'test.c'
// Title		: SPI interface driven E0855-2 SED1530 graphic LCD
// Author		: Marc Meise
// Created		: Oktober 2007
// Version		: Beta
// Target MCU	: Atmel AVR series
//
//
// This code is distributed under the GNU Public License
// which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

/*  Just a small Test-Program to demonstrate the functions of the E0855-2 SED1530 driver.
	The GLCD is connected via SPI.
	The Hardware SPI is used - No Software-SPI available (yet)!
	Development-Environment was a KANDA STK200 (I don't have the STK500) and an ATMega8535 with 8MHz.
	I assume that the demonstration/code should also work on the ATMega16/32 (DIP) without any modifikations after
	compilation with the AVR-GCC compiler.
	Please feel free to modify the driver to your own purpose.

	Disclaimer:I will not take any responsibility or what so ever for the functionality of the E0855-2 SED1530 driver.
	If you wreck anything - it's your own business...
	Modify this code and contribute freely.
*/

// PD7 is congured as a switch to step through the demo.
// I think that the command-functions are fairly self explanatory and are therefore not commented.

//Have fun!


#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <SED1530.h>
#include <font.h>
#include <PICS.h>


int main (void)
{
 DDRB=0xFF;
 PORTB=0xFF;
 DDRD=0x00;
 PORTD=0xFF;
 Init_SED1530();
 SED_clear();
 SED_puts("Press PD7 to");
 SED_newline();
 SED_puts("step through the");
 SED_newline();
 SED_puts("demo of the GLCD");
 SED_set_XY(0,0);
 SED_puts("****************");
 SED_symbol(19,1);

while (1)
{
 while (!( PIND & (1<<PD7)));
 delay();
 while ( PIND & (1<<PD7)) ; 
 delay();
 SED_clear();
 SED_set_XY(3,10);
 SED_puts("This is a DEMO");
 SED_set_XY(2,15); 
 SED_puts("of the GLCD");
 SED_set_XY(1,27);
 SED_puts("E0855-2");
 SED_set_XY(0,27);
 SED_puts("*******");
 SED_symbol(63,1);
 while (!( PIND & (1<<PD7))) ;
 delay();
 while ( PIND & (1<<PD7)) ;
 delay();
 SED_clear_page(0);
 SED_set_XY(0,1);
 SED_puts("A Number: ");
 SED_puts_num(32123);
 SED_symbol(36,1);
 SED_symbol(96,1);
 SED_symbol(95,1);
 SED_symbol(97,1);
 SED_symbol(98,1);
 SED_symbol(63,0);

 while (!( PIND & (1<<PD7))) ;
 delay();
 while ( PIND & (1<<PD7)) ;
 delay();
 SED_showPIC (1);
 SED_symbol(98,0);

 while (!( PIND & (1<<PD7))) ;
 delay();
 while ( PIND & (1<<PD7)) ;
 delay();
 SED_showPIC (2);
 SED_symbol(36,0);
 SED_symbol(96,1);
 SED_symbol(95,1);
 SED_symbol(97,0);
 SED_symbol(98,0);
 SED_symbol(50,1);

 while (!( PIND & (1<<PD7))) ;
 delay();
 while ( PIND & (1<<PD7)) ;
 delay();
 SED_showPIC (4);
 SED_symbol(95,0);
 SED_symbol(50,0);
 SED_symbol(26,1);
 SED_symbol(33,1);
 SED_symbol(40,1);
 SED_symbol(47,1);
 SED_symbol(54,1);

 while (!( PIND & (1<<PD7))) ;
 delay();
 while ( PIND & (1<<PD7)) ;
 delay();
 SED_showPIC (5);
 SED_symbol(96,0);
 SED_symbol(99,1);
 SED_symbol(26,0);
 SED_symbol(33,0);
 SED_symbol(40,0);
 SED_symbol(47,0);
 SED_symbol(54,0);
 SED_set_XY(3,50);
 SED_puts("Lib for");
 SED_set_XY(2,72);
 SED_puts("AVR-");
 SED_set_XY(1,72);
 SED_puts("GCC");

 while (!( PIND & (1<<PD7))) ;
 delay();
 while ( PIND & (1<<PD7)) ;
 delay();
 unsigned char smbl;
 SED_clear();
 for (smbl=19; smbl<=99; smbl++)
 	{
	 SED_symbol(smbl,1);
	}
 SED_set_XY(2,0);
 SED_puts(" All Symbols ON ");
 SED_set_XY(0,74);
 SED_puts("END");

}
 return(0);
}


